@@ -8,20 +8,40 @@ from hacktools import common
 
 
 # Generic extract/repack functions
-def extractNSBMD(infolder, outfolder):
+def extractNSBMD(infolder, outfolder, extension=".nsbmd"):
     common.makeFolder(outfolder)
     common.logMessage("Extracting NSBMD to", outfolder, "...")
-    files = common.getFiles(infolder, ".nsbmd")
+    files = common.getFiles(infolder, extension)
     for file in common.showProgress(files):
         common.logDebug("Processing", file, "...")
         nsbmd = readNSBMD(infolder + file)
         if nsbmd is not None and len(nsbmd.textures) > 0:
             common.makeFolders(outfolder + os.path.dirname(file))
             for texi in range(len(nsbmd.textures)):
-                drawNSBMD(outfolder + file.replace(".nsbmd", "") + "_" + nsbmd.textures[texi].name + ".png", nsbmd, texi)
+                drawNSBMD(outfolder + file.replace(extension, "") + "_" + nsbmd.textures[texi].name + ".png", nsbmd, texi)
     common.logMessage("Done! Extracted", len(files), "files")
 
 
+def repackNSBMD(workfolder, infolder, outfolder, extension=".nsbmd", writefunc=None):
+    common.makeFolder(outfolder)
+    common.logMessage("Repacking NSBMD from", workfolder, "...")
+    files = common.getFiles(infolder, extension)
+    for file in common.showProgress(files):
+        common.logDebug("Processing", file, "...")
+        common.copyFile(infolder + file, outfolder + file)
+        nsbmd = readNSBMD(infolder + file)
+        if nsbmd is not None and len(nsbmd.textures) > 0:
+            fixtrasp = False
+            if writefunc is not None:
+                fixtrasp = writefunc(file, nsbmd)
+            for texi in range(len(nsbmd.textures)):
+                pngname = file.replace(extension, "") + "_" + nsbmd.textures[texi].name + ".png"
+                if os.path.isfile(workfolder + pngname):
+                    common.logDebug(" Repacking", pngname, "...")
+                    writeNSBMD(outfolder + file, nsbmd, texi, workfolder + pngname, fixtrasp)
+    common.logMessage("Done!")
+
+
 def extractIMG(infolder, outfolder, extensions=".NCGR", readfunc=None):
     common.makeFolder(outfolder)
     common.logMessage("Extracting IMG to", outfolder, "...")
@@ -61,20 +81,28 @@ def repackIMG(workfolder, infolder, outfolder, extensions=".NCGR", readfunc=None
             mapfile = file.replace(extension, ".NSCR")
             cellfile = file.replace(extension, ".NCER")
             palettes, image, map, cell, width, height = readNitroGraphic(infolder + palettefile, infolder + file, infolder + mapfile, infolder + cellfile)
-        if image is None:
-            continue
         pngfile = file.replace(extension, ".psd")
         if not os.path.isfile(workfolder + pngfile):
             pngfile = file.replace(extension, ".png")
             if not os.path.isfile(workfolder + pngfile):
-                if clean:
-                    if os.path.isfile(outfolder + file):
-                        os.remove(outfolder + file)
-                    if os.path.isfile(outfolder + mapfile):
-                        os.remove(outfolder + mapfile)
-                    if os.path.isfile(outfolder + cellfile):
-                        os.remove(outfolder + cellfile)
-                continue
+                pngfile = ""
+        if image is None or pngfile == "":
+            if clean:
+                if os.path.isfile(outfolder + file):
+                    os.remove(outfolder + file)
+                if os.path.isfile(outfolder + mapfile):
+                    os.remove(outfolder + mapfile)
+                if os.path.isfile(outfolder + cellfile):
+                    os.remove(outfolder + cellfile)
+            else:
+                common.copyFile(infolder + file, outfolder + file)
+                if os.path.isfile(infolder + mapfile):
+                    common.copyFile(infolder + mapfile, outfolder + mapfile)
+                if os.path.isfile(workfolder + cellfile):
+                    common.copyFile(workfolder + cellfile, outfolder + cellfile)
+                elif os.path.isfile(infolder + cellfile):
+                    common.copyFile(infolder + cellfile, outfolder + cellfile)
+            continue
         common.makeFolders(outfolder + os.path.dirname(file))
         common.copyFile(infolder + file, outfolder + file)
         transptile = False
